function getPrototypeCoreItem(colorScore) {
    let coreMap = {
        0: 'alienevo:prototype_core',
        1: 'alienevo:prototype_core_white',
        2: 'alienevo:prototype_core_light_gray',
        3: 'alienevo:prototype_core_gray',
        4: 'alienevo:prototype_core_black',
        5: 'alienevo:prototype_core_brown',
        6: 'alienevo:prototype_core_red',
        7: 'alienevo:prototype_core_orange',
        8: 'alienevo:prototype_core_yellow',
        9: 'alienevo:prototype_core_green',
        10: 'alienevo:prototype_core_cyan',
        11: 'alienevo:prototype_core_light_blue',
        12: 'alienevo:prototype_core_blue',
        13: 'alienevo:prototype_core_purple',
        14: 'alienevo:prototype_core_magenta',
        15: 'alienevo:prototype_core_pink'
    };
    return coreMap[colorScore] || 'alienevo:prototype_core';
}

function createAndGivePrototypeCore(entity) {
    let MAX_PLAYLISTS = 10;
    let SLOTS_PER_PLAYLIST = 10;
    let OMNITRIX_PROPERTY_KEY = 'omnitrix_cycle';

    let allDNA = {};
    let storedAlienCount = 0;

    let storedItemName = entity.persistentData.getString('storedItemName') || '';

    let alienNamespaceData = {};

    for (let playlist = 1; playlist <= MAX_PLAYLISTS; playlist++) {
        allDNA[playlist] = {};
        for (let slot = 1; slot <= SLOTS_PER_PLAYLIST; slot++) {
            let alienKey = `alienevo.alien_${playlist}_${slot}`;
            let alienId = entity.persistentData.getInt(alienKey);

            if (alienId && alienId !== 0) {
                let alienInfo = global[`alienevo_alien_${alienId}`];
                let alienName = "Unknown";
                let alienNamespace = "alienevo_aliens";

                if (alienInfo) {
                    let alienFullName = alienInfo[0];

                    if (alienFullName.includes(':')) {
                        let parts = alienFullName.split(':');
                        alienNamespace = parts[0];
                        alienName = parts[1];
                    } else {
                        alienName = alienFullName;
                    }

                    alienNamespaceData[alienId] = {
                        namespace: alienNamespace,
                        name: alienName
                    };
                } else {
                    alienName = getAlienName(alienId);
                }

                allDNA[playlist][slot] = {
                    alienId: alienId,
                    alienName: alienName
                };
                storedAlienCount++;
                entity.persistentData.putInt(alienKey, 0);
            }
        }
    }

    let username = entity.getGameProfile().getName();

    let colorProp = palladium.getProperty(entity, 'AlienEvo.Color') || 'default';
    let primaryColorProp = palladium.getProperty(entity, 'AlienEvo.PrimaryColor') || 'default';
    let secondaryColorProp = palladium.getProperty(entity, 'AlienEvo.SecondaryColor') || 'default';

    let omnitrixCycleValue = 0;
    let omnitrixCycleProp = palladium.getProperty(entity, OMNITRIX_PROPERTY_KEY);
    if (omnitrixCycleProp) {
        try {
            omnitrixCycleValue = parseInt(omnitrixCycleProp);
        } catch (e) {
            omnitrixCycleValue = 0;
        }
    }

    let currentCode = entity.persistentData.getString('code') || '000000';
    let currentPosition = entity.persistentData.getString('position') || '0';
    let masterTargetCode = entity.persistentData.getString('masterTargetCode') || '';
    let selfDestructTargetCode = entity.persistentData.getString('selfDestructTargetCode') || '';

    let currentPlaylist = entity.persistentData.getInt('current_playlist') || 1;
    let currentAlienSlot = entity.persistentData.getInt('current_alien_slot') || 1;

    let colorScore = 0;
    if (colorProp && colorProp !== 'default') {
        let colorMap = {
            'white': 1,
            'light_gray': 2,
            'gray': 3,
            'black': 4,
            'brown': 5,
            'red': 6,
            'orange': 7,
            'yellow': 8,
            'green': 9,
            'cyan': 10,
            'light_blue': 11,
            'blue': 12,
            'purple': 13,
            'magenta': 14,
            'pink': 15
        };
        colorScore = colorMap[colorProp] || 0;
    }

    let hasMastercontrol = entity.tags.contains('AlienEvo.MasterControl');
    let hasBaseForm = entity.tags.contains('AlienEvo.BaseForm');

    let prototypeCoreItem = getPrototypeCoreItem(colorScore);

    let displayName = '';
    if (storedItemName && storedItemName !== '') {
        displayName = storedItemName;
    } else {
        displayName = `Prototype Core: All DNA (${storedAlienCount} aliens)`;
    }

    let prototypeCore = Item.of(prototypeCoreItem)
        .withName(Text.of(displayName).darkPurple());

    let nbtData = {
        storedAllDNA: JSON.stringify(allDNA),
        storedAlienCount: storedAlienCount,
        storedAlienNamespaces: JSON.stringify(alienNamespaceData),
        storedCurrentPlaylist: currentPlaylist,
        storedCurrentAlienSlot: currentAlienSlot,
        storedOmnitrixCycle: omnitrixCycleValue,
        storedColor: colorProp,
        storedPrimaryColor: primaryColorProp,
        storedSecondaryColor: secondaryColorProp,
        storedCode: currentCode,
        storedPosition: currentPosition,
        storedMasterTargetCode: masterTargetCode,
        storedSelfDestructTargetCode: selfDestructTargetCode,
        storedMastercontrol: hasMastercontrol,
        storedBaseForm: hasBaseForm
    };
    prototypeCore.setNbt(nbtData);

    entity.give(prototypeCore);

    palladium.setProperty(entity, 'AlienEvo.Color', 'default');
    palladium.setProperty(entity, 'AlienEvo.PrimaryColor', 'default');
    palladium.setProperty(entity, 'AlienEvo.SecondaryColor', 'default');
    palladium.setProperty(entity, OMNITRIX_PROPERTY_KEY, 0);

    entity.persistentData.remove('code');
    entity.persistentData.remove('position');
    entity.persistentData.remove('masterTargetCode');
    entity.persistentData.remove('selfDestructTargetCode');
    entity.server.runCommandSilent(`scoreboard players set ${username} code_position 0`);
    palladium.setProperty(entity, 'code', "000000");

    entity.persistentData.remove('current_playlist');
    entity.persistentData.remove('current_alien_slot');

    entity.tags.remove('AlienEvo.MasterControl');
    entity.tags.remove('AlienEvo.BaseForm');
}

function applyAllDNAFromPrototypeCore(entity) {
    let OMNITRIX_PROPERTY_KEY = 'omnitrix_cycle';

    let mainHandItem = entity.getMainHandItem();
    if (!mainHandItem.id.startsWith('alienevo:prototype_core')) {
        return;
    }

    let itemColor = 'default';
    let itemId = mainHandItem.id;

    if (itemId !== 'alienevo:prototype_core') {
        let parts = itemId.split('_');
        if (parts.length > 2) {
            itemColor = parts[parts.length - 1];

            if (parts.length > 3 && (parts[parts.length - 2] === "light")) {
                itemColor = parts[parts.length - 2] + parts[parts.length - 1];
            } else if (itemColor.startsWith("light")) {
                itemColor = itemColor;
            }
        }
    }

    let username = entity.getGameProfile().getName();
    palladium.setProperty(entity, OMNITRIX_PROPERTY_KEY, 1);

    let itemName = '';
    let nbt = mainHandItem.nbt;

    if (nbt && nbt.display && nbt.display.Name) {
        itemName = nbt.display.Name;
        entity.persistentData.putString('storedItemName', itemName);
    }

    entity.setMainHandItem(Item.of('minecraft:air'));

    if (!nbt) {
        if (itemColor !== 'default') {
            palladium.setProperty(entity, 'AlienEvo.Color', itemColor);
        }

        palladium.setProperty(entity, OMNITRIX_PROPERTY_KEY, 1);
        entity.server.runCommandSilent(`scoreboard players set ${username} omnitrix_cycle 1`);

        giveFirstTenAliensRandomOrder(entity);
        return;
    }

    let storedAllDNAString = nbt.storedAllDNA || '';
    if (!storedAllDNAString) {
        if (itemColor !== 'default') {
            palladium.setProperty(entity, 'AlienEvo.Color', itemColor);
        }

        giveFirstTenAliensRandomOrder(entity);
        return;
    }

    let storedAllDNA;
    try {
        storedAllDNA = JSON.parse(storedAllDNAString);
    } catch (e) {
        if (itemColor !== 'default') {
            palladium.setProperty(entity, 'AlienEvo.Color', itemColor);
        }

        giveFirstTenAliensRandomOrder(entity);
        return;
    }

    let alienNamespaceData = {};
    try {
        if (nbt.storedAlienNamespaces) {
            alienNamespaceData = JSON.parse(nbt.storedAlienNamespaces);
        }
    } catch (e) {
        alienNamespaceData = {};
    }

    for (let playlist in storedAllDNA) {
        for (let slot in storedAllDNA[playlist]) {
            let alienKey = `alienevo.alien_${playlist}_${slot}`;
            let alienData = storedAllDNA[playlist][slot];

            if (alienData && alienData.alienId) {
                entity.persistentData.putInt(alienKey, alienData.alienId);

                let alienId = alienData.alienId;
                if (alienNamespaceData[alienId]) {
                    let alienInfo = [
                        `${alienNamespaceData[alienId].namespace}:${alienNamespaceData[alienId].name}`
                    ];
                    global[`alienevo_alien_${alienId}`] = alienInfo;
                }
            }
        }
    }


    let storedColor = nbt.storedColor || 'default';
    let storedPrimaryColor = nbt.storedPrimaryColor || 'default';
    let storedSecondaryColor = nbt.storedSecondaryColor || 'default';
    let storedOmnitrixCycle = nbt.storedOmnitrixCycle || 1;

    palladium.setProperty(entity, 'AlienEvo.Color', storedColor);
    palladium.setProperty(entity, 'AlienEvo.PrimaryColor', storedPrimaryColor);
    palladium.setProperty(entity, 'AlienEvo.SecondaryColor', storedSecondaryColor);
    palladium.setProperty(entity, OMNITRIX_PROPERTY_KEY, storedOmnitrixCycle);

    let storedCode = nbt.storedCode || '000000';
    let storedPosition = nbt.storedPosition || '0';
    let storedMasterTargetCode = nbt.storedMasterTargetCode || '';
    let storedSelfDestructTargetCode = nbt.storedSelfDestructTargetCode || '';

    entity.persistentData.putString('code', storedCode);
    entity.persistentData.putString('position', storedPosition);
    entity.persistentData.putString('masterTargetCode', storedMasterTargetCode);
    entity.persistentData.putString('selfDestructTargetCode', storedSelfDestructTargetCode);

    entity.server.runCommandSilent(`scoreboard players set ${username} code_position ${parseInt(storedPosition)}`);
    palladium.setProperty(entity, 'code', storedCode);

    let storedCurrentPlaylist = nbt.storedCurrentPlaylist || 1;
    let storedCurrentAlienSlot = nbt.storedCurrentAlienSlot || 1;
    entity.persistentData.putInt('current_playlist', storedCurrentPlaylist);
    entity.persistentData.putInt('current_alien_slot', storedCurrentAlienSlot);

    if (nbt.storedMastercontrol) entity.tags.add('AlienEvo.MasterControl');
    if (nbt.storedBaseForm) entity.tags.add('AlienEvo.BaseForm');
}

function displayDNAInfo(entity) {
    let mainHandItem = entity.getMainHandItem();
    if (!mainHandItem.id.startsWith('alienevo:prototype_core')) {
        return;
    }

    let nbt = mainHandItem.nbt;
    if (!nbt) {
        return;
    }

    let alienNamespaceData = {};
    try {
        if (nbt.storedAlienNamespaces) {
            alienNamespaceData = JSON.parse(nbt.storedAlienNamespaces);
        }
    } catch (e) {
        alienNamespaceData = {};
    }

    let storedAllDNAString = nbt.storedAllDNA || '';
    let storedAlienCount = nbt.storedAlienCount || 0;
    let storedColor = nbt.storedColor || 'default';
    let storedPrimaryColor = nbt.storedPrimaryColor || 'default';
    let storedSecondaryColor = nbt.storedSecondaryColor || 'default';
    let storedOmnitrixCycle = nbt.storedOmnitrixCycle || 0;
    let storedMastercontrol = nbt.storedMastercontrol || false;
    let storedBaseForm = nbt.storedBaseForm || false;

    if (!storedAllDNAString) {
        return;
    }

    let storedAllDNA;
    try {
        storedAllDNA = JSON.parse(storedAllDNAString);
    } catch (e) {
        return;
    }
}

function getAlienName(alienId) {
    let alienInfo = global[`alienevo_alien_${alienId}`];
    if (alienInfo) {
        let fullName = alienInfo[0];
        if (fullName.includes(':')) {
            return fullName.split(':')[1];
        }
        return fullName;
    }

    for (let p = 1; p <= MAX_PLAYLISTS; p++) {
        for (let s = 1; s <= SLOTS_PER_PLAYLIST; s++) {
            let existingData = entity.persistentData.get(`alienevo.alien_name_${p}_${s}`);
            if (existingData && existingData.id === alienId) {
                return existingData.name;
            }
        }
    }
    return `Unknown Alien (ID: ${alienId})`;
}
StartupEvents.registry('palladium:abilities', (event) => {
    event.create('alienevo:create_all_dna_prototype_core')
        .icon(palladium.createItemIcon('alienevo:prototype_core'))
        .addProperty("property", "string", "alienevodnageneration", "please dont change")
        .tick((entity, entry, holder, enabled) => {
            if (enabled) {
                createAndGivePrototypeCore(entity);
            }
        });

    event.create('alienevo:apply_dna_from_prototype_core')
        .icon(palladium.createItemIcon('alienevo:prototype_core'))
        .addProperty("property", "string", "alienevodnaapplication", "please dont change")
        .tick((entity, entry, holder, enabled) => {
            if (enabled) {
                applyAllDNAFromPrototypeCore(entity);
            }
        });

    event.create('alienevo:display_prototype_core_info')
        .icon(palladium.createItemIcon('alienevo:prototype_core'))
        .addProperty("property", "string", "alienevodnainfo", "please dont change")
        .tick((entity, entry, holder, enabled) => {
            if (enabled) {
                displayDNAInfo(entity);
            }
        });
});