StartupEvents.registry('palladium:abilities', (event) => {
  const OMNITRIX_PROPERTY_KEY = 'omnitrix_cycle';
  const TIMER_OBJECTIVE = 'AlienEvo.Timer';
  const MAX_TIMER = 6000;
  const TIMEOUT_WARNING = 5940;
  const TIMEOUT_TIMER = 3000;

  function hasMasterControl(entity) {
    let hasMasterControl = entity.tags.contains("AlienEvo.MasterControl");
    return hasMasterControl;
  }
  function isDead(entity) {
    let isDead = entity.health <= 0 || !entity.isAlive();
    return isDead;
  }
  function handleTimeoutDetransform(entity, alienNumber) {
    let watchType = palladium.getProperty(entity, 'watch') || "prototype";

    if (hasMasterControl(entity)) {
      return false;
    }
    performDetransformation(entity, watchType, alienNumber);
    return true;
  }

  function handleDeathDetransform(entity, alienNumber) {

    let watchType = palladium.getProperty(entity, 'watch') || "prototype";

    if (hasMasterControl(entity)) {
      performDetransformationWithMasterControl(entity, watchType, alienNumber);
    } else {
      performDetransformation(entity, watchType, alienNumber);
    }
    return true;
  }

  function performDetransformationWithMasterControl(entity, watchType, alienNumber) {
    let username = entity.getGameProfile().getName();

    performDetransformationCore(entity, watchType, alienNumber, false);
  }

  function performDetransformation(entity, watchType, alienNumber) {
    let username = entity.getGameProfile().getName();

    palladium.scoreboard.setScore(entity, TIMER_OBJECTIVE, TIMEOUT_TIMER);

    performDetransformationCore(entity, watchType, alienNumber, true);
  }

  function performDetransformationCore(entity, watchType, alienNumber, isTimerModified) {
    let username = entity.getGameProfile().getName();

    let useTimeoutBubble = palladium.getProperty(entity, 'use_timeout_bubble');

    try {
      if (!isDead(entity)) {
        if (entity.persistentData.contains('alienevo.previous_health')) {
          let previousHealth = entity.persistentData.getFloat('alienevo.previous_health');
          entity.health = previousHealth;
        }

        if (entity.persistentData.contains('alienevo.previous_hunger')) {
          let previousHunger = entity.persistentData.getInt('alienevo.previous_hunger');
          entity.foodLevel = previousHunger;
        }
      } else {
      }

      entity.persistentData.remove('alienevo.previous_health');
      entity.persistentData.remove('alienevo.previous_hunger');

      entity.tags.remove("AlienEvo.Transformation");
    } catch (e) {
    }

    let currentAlienNamespace = 'alienevo_aliens';
    let storedNamespace = entity.persistentData.getString('alienevo.current_namespace');
    if (storedNamespace) {
      currentAlienNamespace = storedNamespace;
    } else if (alienNumber > 0) {
      let currentAlienInfo = global[`alienevo_alien_${alienNumber}`];
      if (currentAlienInfo) {
        let currentAlienFullName = currentAlienInfo[0];
        if (currentAlienFullName.includes(':')) {
          let parts = currentAlienFullName.split(':');
          currentAlienNamespace = parts[0];
        }
      }
    }

    let allPowerIds = [];
    try {
      allPowerIds = palladium.powers.getPowerIds(entity);
    } catch (e) {
    }

    for (let powerId of allPowerIds) {
      let powerStr = powerId.toString();

      if (powerStr.startsWith(`${currentAlienNamespace}:`) ||
        powerStr.startsWith('alienevo_aliens:') ||
        powerStr.includes(':')) {

        if (!powerStr.includes("omnitrix") &&
          !powerStr.includes("transform_bubble")) {
          palladium.superpowers.removeSuperpower(entity, powerId);
        }
      }

      if (powerStr === "alienevo:transform_bubble" ||
        powerStr === "alienevo:transform_bubble_tout") {
        palladium.superpowers.removeSuperpower(entity, powerId);
      }
    }

    if (hasMasterControl(entity)) {
      palladium.superpowers.addSuperpower(entity, new ResourceLocation("alienevo:transform_bubble"));
    } else if (useTimeoutBubble === true) {
      palladium.superpowers.addSuperpower(entity, new ResourceLocation("alienevo:transform_bubble_tout"));
    } else {
      palladium.superpowers.addSuperpower(entity, new ResourceLocation("alienevo:transform_bubble"));
    }

    palladium.superpowers.addSuperpower(entity, new ResourceLocation(`alienevo:${watchType}_omnitrix`));
    entity.level.playSound(
      null,
      entity.x,
      entity.y,
      entity.z,
      `alienevo:${watchType}_detransform`,
      entity.getSoundSource(),
      1.0,
      1.0
    );

    if (!isTimerModified && hasMasterControl(entity)) {
    } else {
    }
  }

  event.create('alienevo:omnitrix_timer')
    .icon(palladium.createItemIcon('minecraft:clock'))
    .addProperty("property", "string", OMNITRIX_PROPERTY_KEY, "Internal property key for the Omnitrix cycle")

    .firstTick((entity, entry, holder, enabled) => {
      if (enabled) {
        try {
          palladium.scoreboard.getScore(entity, TIMER_OBJECTIVE, 0);

          if (!entity.tags.contains("AlienEvo.Transformation")) {
            let currentHealth = entity.health || 0;
            if (currentHealth !== undefined && currentHealth !== null) {
              entity.persistentData.putFloat('alienevo.previous_health', currentHealth);
            } else {
            }

            let currentFoodLevel = entity.foodLevel || 0;

            if (currentFoodLevel !== undefined && currentFoodLevel !== null) {
              entity.persistentData.putInt('alienevo.previous_hunger', currentFoodLevel);
            } else {
            }
          } else {
          }

          try {
            entity.server.scheduleInTicks(1, () => {
              try {
                entity.tags.add("AlienEvo.Transformation");

                let maxHealth = entity.getMaxHealth();
                if (maxHealth !== undefined && maxHealth !== null) {
                  entity.health = maxHealth;
                }

                if (entity.foodLevel !== undefined) {
                  entity.foodLevel = 20;
                }

                if (entity.foodSaturationLevel !== undefined) {
                  entity.foodSaturationLevel = 20.0;
                }
              } catch (e) {
              }
            });
          } catch (e) {
            if (entity.getMaxHealth && entity.getMaxHealth() !== undefined) {
              entity.health = entity.getMaxHealth();
            }

            if (entity.foodLevel !== undefined) {
              entity.foodLevel = 20;
            }

            if (entity.foodSaturationLevel !== undefined) {
              entity.foodSaturationLevel = 20.0;
            }
          }
        } catch (e) {
        }
      }
    })

    .tick((entity, entry, holder, enabled) => {
      if (enabled) {
        let username = entity.getGameProfile().getName();

        const propertyKey = entry.getPropertyByName('property');
        let alienNumber = palladium.getProperty(entity, propertyKey);
        let isTransformed = alienNumber > 0;

        if (isTransformed && isDead(entity)) {
          let wasDetransformed = handleDeathDetransform(entity, alienNumber);
          if (wasDetransformed) {
            return;
          }
        }

        if (hasMasterControl(entity)) {
          return;
        }

        palladium.scoreboard.addScore(entity, TIMER_OBJECTIVE, 1);
        let currentTime = palladium.scoreboard.getScore(entity, TIMER_OBJECTIVE, 0);

        let watchType = palladium.getProperty(entity, 'watch') || "prototype";

        if (isTransformed && currentTime == TIMEOUT_WARNING) {
          entity.level.playSound(
            null, 
            entity.x, 
            entity.y, 
            entity.z, 
            `alienevo:${watchType}_timeout`, 
            entity.getSoundSource(), 
            1.0, 
            1.0
        );
        }

        if (!isTransformed && currentTime > 0) {
          palladium.scoreboard.setScore(entity, TIMER_OBJECTIVE, 0);
        }

        if (isTransformed && currentTime >= MAX_TIMER) {
          handleTimeoutDetransform(entity, alienNumber);
        }
      }
    })

    .lastTick((entity, entry, holder, enabled) => {
      try {
        if (!isDead(entity)) {
          if (entity.persistentData.contains('alienevo.previous_health')) {
            let previousHealth = entity.persistentData.getFloat('alienevo.previous_health');
            entity.health = previousHealth;
          }

          if (entity.persistentData.contains('alienevo.previous_hunger')) {
            let previousHunger = entity.persistentData.getInt('alienevo.previous_hunger');
            entity.foodLevel = previousHunger;
          }
        }

        entity.persistentData.remove('alienevo.previous_health');
        entity.persistentData.remove('alienevo.previous_hunger');
      } catch (e) {
      }
    });
});