global.sound = (entity, soundId, volume, pitch, shift) => {
    entity.level.runCommandSilent(`playsound ${soundId} player @p ${entity.x} ${entity.y} ${entity.z} ${volume} ${pitch - shift/2 + Math.random()*shift}`);
};

global.applyKnockback = (entity, target, strength) => {
    const dx = target.x - entity.x;
    const dz = target.z - entity.z;
    const distance = Math.sqrt(dx * dx + dz * dz);
    
    if (distance > 0) {
        const multiplier = strength / distance;
        target.addMotion(
            dx * multiplier,
            0.4,
            dz * multiplier
        );
    }
};

function effectPack(reference, entity) {
    Object.keys(reference).forEach(effect => {
        const data = reference[effect];
        entity.potionEffects.add(effect, data.time, data.amp || 0, data.ambient || false, data.particle || true);
    });
}

function getCooldownTime(entity) {
    return entity.persistentData.getInt('totemCooldown') || 0;
}

function setCooldownTime(entity, time) {
    entity.persistentData.putInt('totemCooldown', time);
}

function isOnCooldown(entity) {
    const currentTime = entity.level.time;
    const cooldownTime = getCooldownTime(entity);
    return currentTime < cooldownTime;
}

function getRemainingCooldown(entity) {
    const currentTime = entity.level.time;
    const cooldownTime = getCooldownTime(entity);
    const remainingTicks = cooldownTime - currentTime;
    return Math.max(0, Math.ceil(remainingTicks / 20));
}

const totemEffects = {
    'minecraft:fire_resistance': {time: 40*20, ambient: true, particle: false},
    'minecraft:regeneration': {time: 45*20, amp: 1, ambient: true, particle: false},
    'minecraft:resistance': {time: 5*20, amp: 1, ambient: true, particle: false}
};

const users = ['minecraft:player'];

users.forEach(key => {
    EntityEvents.death(key, e=> {
        const bypasses = ['outOfWorld'];
        const knockbackRadius = 10;
        const knockbackStrength = 3;
        
        const {entity, source} = e;
        
        if (isOnCooldown(entity)) {
            return;
        }

        if (!abilityUtil.hasPower(entity, "alienevo:prototype_omnitrix")) {
            return;
        }
        
        if (!palladium.abilities.isUnlocked(entity, new ResourceLocation("alienevo:prototype_omnitrix"), "failsafe_unlock")) {
            return;
        }
        
        if (bypasses.includes(source.type)) {
            return;
        }

        const cooldownDuration = 18000;
        setCooldownTime(entity, entity.level.time + cooldownDuration);
        
        palladium.scoreboard.setScore(entity, "AlienEvo.Timer", 0);
        
        const box = AABB.ofSize(entity.position(), knockbackRadius * 2, knockbackRadius * 2, knockbackRadius * 2);
        const nearbyEntities = entity.level.getEntities(entity, box);
        
        nearbyEntities.forEach(target => {
            if (target !== entity && target.type !== "minecraft:item" && 
                target.type !== "minecraft:item_frame" && 
                target.type !== "minecraft:glow_item_frame" && 
                target.type !== "minecraft:armor_stand") {
                global.applyKnockback(entity, target, knockbackStrength);
            }
        });

        superpowerUtil.addSuperpower(entity, new ResourceLocation(`alienevo:transform_bubble`));
        global.sound(entity, 'item.totem.use', 0.3, 1.8, 0.2);
        global.sound(entity, 'alienevo:prototype_failsafe', 1.0, 1.0, 0.0);
        
        effectPack(totemEffects, entity);
        entity.setHealth(1.0);
        e.cancel();
    })
});