// Create a global registry to track placed pods
global.playerPlacedPods = global.playerPlacedPods || new Set();

// Add a counter to track pod openings
global.podOpenCount = global.podOpenCount || 0;

// Set a specific rare chance - e.g., 1 in 10,000 pods will have the omnitrix
const OMNITRIX_RARITY = 1000;

BlockEvents.rightClicked('alienevo:galvan_pod', event => {
    let blockEntity = event.block.entity;
    if (!blockEntity) return;
    
    // Get the position as a string for tracking
    const pos = `${event.block.x},${event.block.y},${event.block.z},${event.block.level.dimension}`;
    
    // If this pod was placed by a player, don't generate loot
    if (global.playerPlacedPods.has(pos)) return;

    // Increment the pod open counter
    global.podOpenCount++;
    
    // Determine if this is a special pod that will contain the omnitrix
    // This guarantees exactly 1 in OMNITRIX_RARITY pods will have it
    const hasOmnitrix = (global.podOpenCount % OMNITRIX_RARITY === 0);

    const itemGroups = {
        shards: {
            maxTotal: 6,
            weight: 10,
            items: [
                { item: 'alienevo:upgrade_shards', weight: 4, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_black', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_green', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_brown', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_cyan', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_magenta', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_light_gray', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_gray', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_pink', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_purple', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_light_blue', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_blue', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_red', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_orange', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_white', weight: 2, maxStack: 3 },
                { item: 'alienevo:upgrade_shards_yellow', weight: 2, maxStack: 3 }
            ]
        },
        crystals: {
            maxTotal: 2,
            weight: 4,
            items: [
                { item: 'alienevo:upgrade_crystal', weight: 0.5, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_black', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_green', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_brown', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_cyan', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_magenta', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_light_gray', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_gray', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_pink', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_purple', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_light_blue', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_blue', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_red', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_orange', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_white', weight: 0.25, maxStack: 1 },
                { item: 'alienevo:upgrade_crystal_yellow', weight: 0.25, maxStack: 1 }
            ]
        },
        materials: {
            maxTotal: 8,
            weight: 15,
            items: [
                { item: 'minecraft:iron_ingot', weight: 6, maxStack: 2 },
                { item: 'palladium:lead_ingot', weight: 5, maxStack: 2 },
                { item: 'palladium:raw_titanium', weight: 0.5, maxStack: 1 },
                { item: 'alienevo:galvan_scrap', weight: 16, maxStack: 4 },
                { item: 'alienevo:galvan_metal', weight: 8, maxStack: 2 }
            ]
        }
    };

    const groupCounts = {};
    for (let groupName in itemGroups) {
        groupCounts[groupName] = 0;
    }

    function pickRandomGroup() {
        // Calculate total weight of all groups
        let totalWeight = 0;
        for (const groupName in itemGroups) {
            totalWeight += itemGroups[groupName].weight;
        }

        // Use a random value between 0 and totalWeight
        const random = Math.random() * totalWeight;
        
        // Accumulate weights to find the selected group
        let accumulatedWeight = 0;
        for (const groupName in itemGroups) {
            accumulatedWeight += itemGroups[groupName].weight;
            if (random <= accumulatedWeight) {
                return [groupName, itemGroups[groupName]];
            }
        }
        
        // Fallback to first group (should rarely happen)
        const firstGroup = Object.keys(itemGroups)[0];
        return [firstGroup, itemGroups[firstGroup]];
    }

    function pickRandomItem(groupName, group) {
        // Calculate total weight of all items in the group
        let totalWeight = 0;
        for (const itemConfig of group.items) {
            totalWeight += itemConfig.weight;
        }
        
        // Use a random value between 0 and totalWeight
        const random = Math.random() * totalWeight;
        
        // Accumulate weights to find the selected item
        let accumulatedWeight = 0;
        for (const itemConfig of group.items) {
            accumulatedWeight += itemConfig.weight;
            if (random <= accumulatedWeight) {
                const remainingItems = group.maxTotal - groupCounts[groupName];
                const possibleStack = Math.min(itemConfig.maxStack, remainingItems);
                if (possibleStack <= 0) return null;
                
                const stackSize = Math.floor(Math.random() * possibleStack) + 1;
                groupCounts[groupName] += stackSize;
                
                return {
                    item: itemConfig.item,
                    stackSize: stackSize
                };
            }
        }
        
        // Fallback (should rarely happen)
        return null;
    }

    // Determine how many inventory slots to fill (between 5 and 10)
    const slotsToFill = Math.floor(Math.random() * 6) + 5;
    const usedSlots = new Set();
    
    // If this is a special pod with the omnitrix, reserve a slot for it
    let omnitrixSlot = -1;
    if (hasOmnitrix) {
        omnitrixSlot = Math.floor(Math.random() * 27);
        usedSlots.add(omnitrixSlot);
    }
    
    // Randomly select which inventory slots to use for regular items
    while (usedSlots.size < slotsToFill) {
        let slot = Math.floor(Math.random() * 27);
        usedSlots.add(slot);
    }

    // Fill the selected slots with items
    usedSlots.forEach(slot => {
        // If this is the omnitrix slot, add the omnitrix
        if (hasOmnitrix && slot === omnitrixSlot) {
            blockEntity.inventory.setStackInSlot(slot, Item.of('alienevo:prototype_omnitrix', 1));
        } else {
            // Otherwise, add a regular item
            const [groupName, group] = pickRandomGroup();
            const selectedItem = pickRandomItem(groupName, group);
            
            if (selectedItem) {
                blockEntity.inventory.setStackInSlot(slot, Item.of(selectedItem.item, selectedItem.stackSize));
            }
        }
    });

    // Mark this pod as filled
    global.playerPlacedPods.add(pos);
});

BlockEvents.placed('alienevo:galvan_pod', event => {
    const pos = `${event.block.x},${event.block.y},${event.block.z},${event.block.level.dimension}`;
    global.playerPlacedPods.add(pos);
});