ServerEvents.commandRegistry(event => {
    const { commands: Commands, arguments: Arguments } = event;
 
    const getPlayerAliensList = (context, builder) => {
        const player = context.source.player;
        if (!player) return builder.buildFuture();
 
        for (let p = 1; p <= 10; p++) {
            for (let s = 1; s <= 10; s++) {
                let key = `alienevo.alien_${p}_${s}`;
                let value = player.persistentData.get(key);
                
                if (value && value > 0) {
                    let alienKey = `alienevo_alien_${value}`;
                    if (global[alienKey]) {
                        let alienName = global[alienKey][0];
                        // Check if the alien name already includes a namespace
                        if (!alienName.includes(':')) {
                            // If no namespace, add the default one
                            alienName = `alienevo_aliens:${alienName}`;
                        }
                        builder.suggest(alienName);
                    }
                }
            }
        }
        return builder.buildFuture();
    };
 
    const getKeysList = (context, builder) => {
        builder.suggest("QUICKCHANGE01");
        builder.suggest("QUICKCHANGE02");
        builder.suggest("QUICKCHANGE03");
        builder.suggest("QUICKCHANGE04");
        builder.suggest("QUICKCHANGE05");
        builder.suggest("QUICKCHANGE06");
        builder.suggest("QUICKCHANGE07");
        builder.suggest("QUICKCHANGE08");
        builder.suggest("QUICKCHANGE09");
        builder.suggest("QUICKCHANGE10");
        return builder.buildFuture();
    };
 
    event.register(
        Commands.literal("bindtransform")
        .requires(src => src.hasPermission(0))
        .then(Commands.argument('alien', Arguments.RESOURCE_LOCATION.create(event))
            .suggests(getPlayerAliensList)
            .then(Commands.argument('key', Arguments.STRING.create(event))
                .suggests(getKeysList)
                .executes(ctx => {
                    const player = ctx.source.player;
                    const alienResource = Arguments.RESOURCE_LOCATION.getResult(ctx, "alien");
                    const alienName = alienResource.toString(); // Get the full resource location as string
                    const keyName = Arguments.STRING.getResult(ctx, "key");
 
                    // Save to player's persistent data
                    player.persistentData.putString(`alienevo.bound.${keyName}`, alienName);
                    
                    console.log(`[QuickChange] Bound ${alienName} to ${keyName} for player ${player.getName()}`);
                    ctx.source.sendSuccess(Text.of(`Bound ${alienName} to ${keyName}`), true);
                    return 1;
                })
            )
        )
    );
});

// Utility function to store armor
function storePlayerArmor(player) {
    let armorSlots = ['head', 'chest', 'legs', 'feet'];
    
    // Check if player has any armor before proceeding
    let hasAnyArmor = armorSlots.some(slot => {
        let armorItem = player[slot + 'ArmorItem'];
        return armorItem && armorItem.id !== 'minecraft:air';
    });
    
    // Only proceed if player has at least one piece of armor
    if (hasAnyArmor) {
        let hasStoredArmor = false;
        
        // Store each armor piece
        armorSlots.forEach(slot => {
            let armorItem = player[slot + 'ArmorItem'];
            if (armorItem && armorItem.id !== 'minecraft:air') {
                hasStoredArmor = true;
                console.log(`[QuickChange] Storing ${slot} armor for player ${player.getName()}: ${armorItem.id}`);
                player.persistentData.put(slot + 'ArmorStore', armorItem);
                player.server.runCommandSilent(`item replace entity ${player.getGameProfile().getName()} armor.${slot} with air`);
            }
        });
        
        if (hasStoredArmor) {
            player.persistentData.putBoolean('hasStoredArmor', true);
        }
        return hasStoredArmor;
    }
    
    return false;
}

// Utility function to retrieve armor
function retrievePlayerArmor(player) {
    if (!player.persistentData.getBoolean('hasStoredArmor')) {
        console.log(`[QuickChange] No stored armor found for player ${player.getName()}`);
        return false;
    }
    
    let armorSlots = ['head', 'chest', 'legs', 'feet'];
    
    // Retrieve each stored armor piece
    armorSlots.forEach(slot => {
        if (player.persistentData.contains(slot + 'ArmorStore')) {
            let storedArmor = player.persistentData.get(slot + 'ArmorStore');
            
            // Get the complete NBT string
            let nbtString = '';
            if (storedArmor.tag) {
                nbtString = storedArmor.tag.toString();
            }
            
            let command = `item replace entity ${player.getGameProfile().getName()} armor.${slot} with ${storedArmor.id}${nbtString}`;
            console.log(`[QuickChange] Retrieving ${slot} armor for player ${player.getName()}: ${storedArmor.id}`);
            player.server.runCommandSilent(command);
            player.persistentData.remove(slot + 'ArmorStore');
        }
    });
    
    player.persistentData.remove('hasStoredArmor');
    return true;
}
NetworkEvents.dataReceived('quickchange_key_pressed', event => {
    let player = event.player;
    let keyName = event.data.keyName;
    
    // Constants to match other scripts
    const OMNITRIX_PROPERTY_KEY = 'omnitrix_cycle';
    
    // Initial validation checks
    if (!player || player.type != 'minecraft:player') {
        return;
    }
    
    if (!player.tags.contains("AlienEvo.MasterControl")) {
        return;
    }
    
    let alienFullName = player.persistentData.getString(`alienevo.bound.${keyName}`);
    if (!alienFullName) {
        return;
    }
    
    // Parse the alien name for namespace support
    let alienNamespace = 'alienevo_aliens'; // Default namespace
    let alienPath = alienFullName;
    
    // Parse namespace if the alien name contains a colon
    if (alienFullName.includes(':')) {
        let nameParts = alienFullName.split(':');
        alienNamespace = nameParts[0];
        alienPath = nameParts[1];
    }
    
    // Check if the alien exists in any of the player's playlists
    let hasAlienInPlaylist = false;
    let alienNumber = -1;
    
    // Find the alien's number based on name (accounting for namespaces)
    for (let i = 1; i <= 300; i++) {
        if (global[`alienevo_alien_${i}`]) {
            let globalAlienName = global[`alienevo_alien_${i}`][0];
            
            // Compare with full name or just the path part depending on what's stored
            if (globalAlienName.toLowerCase() === alienFullName.toLowerCase() || 
                (alienFullName.includes(':') && globalAlienName.toLowerCase() === alienPath.toLowerCase())) {
                alienNumber = i;
                break;
            }
        }
    }
    
    // If alien wasn't found in the database, return
    if (alienNumber === -1) {
        return;
    }
    
    // Check if the alien exists in any of the player's playlists
    for (let playlist = 1; playlist <= 10; playlist++) {
        for (let slot = 1; slot <= 10; slot++) {
            let alienInSlot = player.persistentData.getInt(`alienevo.alien_${playlist}_${slot}`);
            if (alienInSlot === alienNumber) {
                hasAlienInPlaylist = true;
                break;
            }
        }
        if (hasAlienInPlaylist) break;
    }
    
    // Only proceed if the player has the alien in their playlist
    if (!hasAlienInPlaylist) {
        return;
    }
    
    try {
        // Force a synchronous update of the property to ensure it's current
        player.persistentData.putInt('alien_transition_lock', 1);
        
        // Get current omnitrix cycle property - contains the current alien number
        let currentAlienNumber = palladium.getProperty(player, OMNITRIX_PROPERTY_KEY);
        
        // Get watch type from property (for sounds and visuals)
        let watchType = palladium.getProperty(player, 'watch') || "prototype";
        
        // Get timeout bubble preference
        let useTimeoutBubble = palladium.getProperty(player, 'use_timeout_bubble');
        
        // Get target alien power ID
        let targetAlienPowerID = `${alienNamespace}:${alienPath}`;
        
        // If we're in the middle of a transformation (lock active), exit early
        if (player.persistentData.getInt('alien_transition_lock') > 1) {
            player.persistentData.putInt('alien_transition_lock', 0);
            return;
        }
        
        // Get all current powers
        let allPowerIds = [];
        try {
            allPowerIds = palladium.powers.getPowerIds(player);
        } catch (e) {}
        
        // Check for bubble
        let hasBubble = allPowerIds.some(id => id.toString() === "alienevo:transform_bubble");
        
        // If transformed into any alien, detransform first
        if (hasBubble || currentAlienNumber > 0) {
            // Find the current alien's namespace if we already have an alien
            let currentAlienNamespace = 'alienevo_aliens'; // Default
            
            if (currentAlienNumber > 0) {
                let currentAlienInfo = global[`alienevo_alien_${currentAlienNumber}`];
                if (currentAlienInfo) {
                    let currentAlienFullName = currentAlienInfo[0];
                    
                    // Parse namespace if the alien name contains a colon
                    if (currentAlienFullName.includes(':')) {
                        let parts = currentAlienFullName.split(':');
                        currentAlienNamespace = parts[0];
                    }
                }
            }
            
            // Using same logic as detransform script
            // Remove specific powers using the Palladium API
            for (let powerId of allPowerIds) {
                let powerStr = powerId.toString();
                // Remove powers from relevant namespaces
                if (powerStr.startsWith(`${currentAlienNamespace}:`) || 
                    (currentAlienNamespace !== 'alienevo_aliens' && powerStr.startsWith('alienevo_aliens:')) ||
                    (alienNamespace !== currentAlienNamespace && alienNamespace !== 'alienevo_aliens' && powerStr.startsWith(`${alienNamespace}:`))) {
                    palladium.superpowers.removeSuperpower(player, powerId);
                }
            }
            
            // Remove any transform bubbles
            if (hasBubble) {
                palladium.superpowers.removeSuperpower(player, new ResourceLocation("alienevo:transform_bubble"));
                palladium.superpowers.removeSuperpower(player, new ResourceLocation("alienevo:transform_bubble_tout"));
            }
            
            // Reset the property
            palladium.setProperty(player, OMNITRIX_PROPERTY_KEY, 0);
            
            // Add back the omnitrix with proper watch type
            palladium.superpowers.addSuperpower(player, new ResourceLocation(`alienevo:${watchType}_omnitrix`));
            
            // Only retrieve armor if we're fully detransforming (not changing aliens)
            if (alienNumber === -1) {
                retrievePlayerArmor(player);
            }
        }
        
        // Check if the player already has any alien powers and if they already have the specific alien power
        let isAlreadyTransformed = false;
        let hasTargetAlienPower = false;
        
        try {
            // Check if player has the specific alien power we're trying to transform into
            hasTargetAlienPower = allPowerIds.some(id => {
                return id.toString() === targetAlienPowerID;
            });
            
            // Check if player has any powers from the alien namespaces
            isAlreadyTransformed = allPowerIds.some(id => {
                let powerStr = id.toString();
                return powerStr.startsWith("alienevo_aliens:") || 
                       (alienNamespace !== 'alienevo_aliens' && powerStr.startsWith(`${alienNamespace}:`));
            });
        } catch (e) {}
        
        // Only store armor if not already transformed
        if (!isAlreadyTransformed) {
            storePlayerArmor(player);
        }
        
        // Set the property to track the current alien and increment the lock
        palladium.setProperty(player, OMNITRIX_PROPERTY_KEY, alienNumber);
        player.persistentData.putInt('alien_transition_lock', 2); // Mark transformation as in progress
        
        // Remove the current omnitrix (any type)
        let omnitrixResourceLocation = new ResourceLocation(`alienevo:${watchType}_omnitrix`);
        palladium.superpowers.removeSuperpower(player, omnitrixResourceLocation);
        
        // Transform into alien with namespace support
        palladium.superpowers.addSuperpower(player, new ResourceLocation(`${alienNamespace}:${alienPath}`));
        
        // Only play transform sound and add transform bubble if not transforming into the same alien
        if (!hasTargetAlienPower) {
            player.runCommandSilent(`playsound alienevo:${watchType}_transform player @a[distance=..5] ~ ~ ~ 1 1 1`);
            palladium.superpowers.addSuperpower(player, new ResourceLocation("alienevo:transform_bubble"));
        }
        
        // Reset the lock when transformation is complete
        player.persistentData.putInt('alien_transition_lock', 0);
    } catch (e) {
        // Ensure lock is reset even on error
        player.persistentData.putInt('alien_transition_lock', 0);
    }
});
